{% extends "socialaccount/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Login Cancelled" %}{% endblock %}

{% block content %}

<h1>{% trans "Login Cancelled" %}</h1>

{% url 'account_login' as login_url %}

<p>{% blocktrans %}You decided to cancel logging in to our site using one of your existing accounts. If this was a mistake, please proceed to <a href="{{login_url}}">sign in</a>.{% endblocktrans %}</p>

{% endblock %}
